use rgb::RGB8;


pub struct TrueColor {
    pub r: u8,
    pub g: u8,
    pub b: u8,
}

pub fn truecolor_from_avt(color: avt::Color) -> TrueColor {
    match color {
        avt::Color::RGB(rgb8) => {
            return TrueColor {
                r: rgb8.r,
                g: rgb8.g,
                b: rgb8.b,
            }
        }
        avt::Color::Indexed(u8) => {
            let rgb8 = PALETTE[usize::from(u8)];
            return TrueColor {
                r: rgb8.r,
                g: rgb8.g,
                b: rgb8.b,
            };
        }
    }
}

pub fn indexedcolor_from_avt(color: avt::Color) -> u8 {
    match color {
        avt::Color::RGB(_) => {
            eprintln!(
                "warning, truecolor to indexedcolor conversion, not implemented, always returning 0, could find nearest"
            );
            0
        }
        avt::Color::Indexed(u8) => u8,
    }
}

// converted from https://gist.github.com/MicahElliott/719710
pub static PALETTE: [RGB8; 256] = [
    RGB8{r: 0, g: 0, b: 0},
    RGB8{r: 128, g: 0, b: 0},
    RGB8{r: 0, g: 128, b: 0},
    RGB8{r: 128, g: 128, b: 0},
    RGB8{r: 0, g: 0, b: 128},
    RGB8{r: 128, g: 0, b: 128},
    RGB8{r: 0, g: 128, b: 128},
    RGB8{r: 192, g: 192, b: 192},
    RGB8{r: 128, g: 128, b: 128},
    RGB8{r: 255, g: 0, b: 0},
    RGB8{r: 0, g: 255, b: 0},
    RGB8{r: 255, g: 255, b: 0},
    RGB8{r: 0, g: 0, b: 255},
    RGB8{r: 255, g: 0, b: 255},
    RGB8{r: 0, g: 255, b: 255},
    RGB8{r: 255, g: 255, b: 255},
    RGB8{r: 0, g: 0, b: 0},
    RGB8{r: 0, g: 0, b: 95},
    RGB8{r: 0, g: 0, b: 135},
    RGB8{r: 0, g: 0, b: 175},
    RGB8{r: 0, g: 0, b: 215},
    RGB8{r: 0, g: 0, b: 255},
    RGB8{r: 0, g: 95, b: 0},
    RGB8{r: 0, g: 95, b: 95},
    RGB8{r: 0, g: 95, b: 135},
    RGB8{r: 0, g: 95, b: 175},
    RGB8{r: 0, g: 95, b: 215},
    RGB8{r: 0, g: 95, b: 255},
    RGB8{r: 0, g: 135, b: 0},
    RGB8{r: 0, g: 135, b: 95},
    RGB8{r: 0, g: 135, b: 135},
    RGB8{r: 0, g: 135, b: 175},
    RGB8{r: 0, g: 135, b: 215},
    RGB8{r: 0, g: 135, b: 255},
    RGB8{r: 0, g: 175, b: 0},
    RGB8{r: 0, g: 175, b: 95},
    RGB8{r: 0, g: 175, b: 135},
    RGB8{r: 0, g: 175, b: 175},
    RGB8{r: 0, g: 175, b: 215},
    RGB8{r: 0, g: 175, b: 255},
    RGB8{r: 0, g: 215, b: 0},
    RGB8{r: 0, g: 215, b: 95},
    RGB8{r: 0, g: 215, b: 135},
    RGB8{r: 0, g: 215, b: 175},
    RGB8{r: 0, g: 215, b: 215},
    RGB8{r: 0, g: 215, b: 255},
    RGB8{r: 0, g: 255, b: 0},
    RGB8{r: 0, g: 255, b: 95},
    RGB8{r: 0, g: 255, b: 135},
    RGB8{r: 0, g: 255, b: 175},
    RGB8{r: 0, g: 255, b: 215},
    RGB8{r: 0, g: 255, b: 255},
    RGB8{r: 95, g: 0, b: 0},
    RGB8{r: 95, g: 0, b: 95},
    RGB8{r: 95, g: 0, b: 135},
    RGB8{r: 95, g: 0, b: 175},
    RGB8{r: 95, g: 0, b: 215},
    RGB8{r: 95, g: 0, b: 255},
    RGB8{r: 95, g: 95, b: 0},
    RGB8{r: 95, g: 95, b: 95},
    RGB8{r: 95, g: 95, b: 135},
    RGB8{r: 95, g: 95, b: 175},
    RGB8{r: 95, g: 95, b: 215},
    RGB8{r: 95, g: 95, b: 255},
    RGB8{r: 95, g: 135, b: 0},
    RGB8{r: 95, g: 135, b: 95},
    RGB8{r: 95, g: 135, b: 135},
    RGB8{r: 95, g: 135, b: 175},
    RGB8{r: 95, g: 135, b: 215},
    RGB8{r: 95, g: 135, b: 255},
    RGB8{r: 95, g: 175, b: 0},
    RGB8{r: 95, g: 175, b: 95},
    RGB8{r: 95, g: 175, b: 135},
    RGB8{r: 95, g: 175, b: 175},
    RGB8{r: 95, g: 175, b: 215},
    RGB8{r: 95, g: 175, b: 255},
    RGB8{r: 95, g: 215, b: 0},
    RGB8{r: 95, g: 215, b: 95},
    RGB8{r: 95, g: 215, b: 135},
    RGB8{r: 95, g: 215, b: 175},
    RGB8{r: 95, g: 215, b: 215},
    RGB8{r: 95, g: 215, b: 255},
    RGB8{r: 95, g: 255, b: 0},
    RGB8{r: 95, g: 255, b: 95},
    RGB8{r: 95, g: 255, b: 135},
    RGB8{r: 95, g: 255, b: 175},
    RGB8{r: 95, g: 255, b: 215},
    RGB8{r: 95, g: 255, b: 255},
    RGB8{r: 135, g: 0, b: 0},
    RGB8{r: 135, g: 0, b: 95},
    RGB8{r: 135, g: 0, b: 135},
    RGB8{r: 135, g: 0, b: 175},
    RGB8{r: 135, g: 0, b: 215},
    RGB8{r: 135, g: 0, b: 255},
    RGB8{r: 135, g: 95, b: 0},
    RGB8{r: 135, g: 95, b: 95},
    RGB8{r: 135, g: 95, b: 135},
    RGB8{r: 135, g: 95, b: 175},
    RGB8{r: 135, g: 95, b: 215},
    RGB8{r: 135, g: 95, b: 255},
    RGB8{r: 135, g: 135, b: 0},
    RGB8{r: 135, g: 135, b: 95},
    RGB8{r: 135, g: 135, b: 135},
    RGB8{r: 135, g: 135, b: 175},
    RGB8{r: 135, g: 135, b: 215},
    RGB8{r: 135, g: 135, b: 255},
    RGB8{r: 135, g: 175, b: 0},
    RGB8{r: 135, g: 175, b: 95},
    RGB8{r: 135, g: 175, b: 135},
    RGB8{r: 135, g: 175, b: 175},
    RGB8{r: 135, g: 175, b: 215},
    RGB8{r: 135, g: 175, b: 255},
    RGB8{r: 135, g: 215, b: 0},
    RGB8{r: 135, g: 215, b: 95},
    RGB8{r: 135, g: 215, b: 135},
    RGB8{r: 135, g: 215, b: 175},
    RGB8{r: 135, g: 215, b: 215},
    RGB8{r: 135, g: 215, b: 255},
    RGB8{r: 135, g: 255, b: 0},
    RGB8{r: 135, g: 255, b: 95},
    RGB8{r: 135, g: 255, b: 135},
    RGB8{r: 135, g: 255, b: 175},
    RGB8{r: 135, g: 255, b: 215},
    RGB8{r: 135, g: 255, b: 255},
    RGB8{r: 175, g: 0, b: 0},
    RGB8{r: 175, g: 0, b: 95},
    RGB8{r: 175, g: 0, b: 135},
    RGB8{r: 175, g: 0, b: 175},
    RGB8{r: 175, g: 0, b: 215},
    RGB8{r: 175, g: 0, b: 255},
    RGB8{r: 175, g: 95, b: 0},
    RGB8{r: 175, g: 95, b: 95},
    RGB8{r: 175, g: 95, b: 135},
    RGB8{r: 175, g: 95, b: 175},
    RGB8{r: 175, g: 95, b: 215},
    RGB8{r: 175, g: 95, b: 255},
    RGB8{r: 175, g: 135, b: 0},
    RGB8{r: 175, g: 135, b: 95},
    RGB8{r: 175, g: 135, b: 135},
    RGB8{r: 175, g: 135, b: 175},
    RGB8{r: 175, g: 135, b: 215},
    RGB8{r: 175, g: 135, b: 255},
    RGB8{r: 175, g: 175, b: 0},
    RGB8{r: 175, g: 175, b: 95},
    RGB8{r: 175, g: 175, b: 135},
    RGB8{r: 175, g: 175, b: 175},
    RGB8{r: 175, g: 175, b: 215},
    RGB8{r: 175, g: 175, b: 255},
    RGB8{r: 175, g: 215, b: 0},
    RGB8{r: 175, g: 215, b: 95},
    RGB8{r: 175, g: 215, b: 135},
    RGB8{r: 175, g: 215, b: 175},
    RGB8{r: 175, g: 215, b: 215},
    RGB8{r: 175, g: 215, b: 255},
    RGB8{r: 175, g: 255, b: 0},
    RGB8{r: 175, g: 255, b: 95},
    RGB8{r: 175, g: 255, b: 135},
    RGB8{r: 175, g: 255, b: 175},
    RGB8{r: 175, g: 255, b: 215},
    RGB8{r: 175, g: 255, b: 255},
    RGB8{r: 215, g: 0, b: 0},
    RGB8{r: 215, g: 0, b: 95},
    RGB8{r: 215, g: 0, b: 135},
    RGB8{r: 215, g: 0, b: 175},
    RGB8{r: 215, g: 0, b: 215},
    RGB8{r: 215, g: 0, b: 255},
    RGB8{r: 215, g: 95, b: 0},
    RGB8{r: 215, g: 95, b: 95},
    RGB8{r: 215, g: 95, b: 135},
    RGB8{r: 215, g: 95, b: 175},
    RGB8{r: 215, g: 95, b: 215},
    RGB8{r: 215, g: 95, b: 255},
    RGB8{r: 215, g: 135, b: 0},
    RGB8{r: 215, g: 135, b: 95},
    RGB8{r: 215, g: 135, b: 135},
    RGB8{r: 215, g: 135, b: 175},
    RGB8{r: 215, g: 135, b: 215},
    RGB8{r: 215, g: 135, b: 255},
    RGB8{r: 215, g: 175, b: 0},
    RGB8{r: 215, g: 175, b: 95},
    RGB8{r: 215, g: 175, b: 135},
    RGB8{r: 215, g: 175, b: 175},
    RGB8{r: 215, g: 175, b: 215},
    RGB8{r: 215, g: 175, b: 255},
    RGB8{r: 215, g: 215, b: 0},
    RGB8{r: 215, g: 215, b: 95},
    RGB8{r: 215, g: 215, b: 135},
    RGB8{r: 215, g: 215, b: 175},
    RGB8{r: 215, g: 215, b: 215},
    RGB8{r: 215, g: 215, b: 255},
    RGB8{r: 215, g: 255, b: 0},
    RGB8{r: 215, g: 255, b: 95},
    RGB8{r: 215, g: 255, b: 135},
    RGB8{r: 215, g: 255, b: 175},
    RGB8{r: 215, g: 255, b: 215},
    RGB8{r: 215, g: 255, b: 255},
    RGB8{r: 255, g: 0, b: 0},
    RGB8{r: 255, g: 0, b: 95},
    RGB8{r: 255, g: 0, b: 135},
    RGB8{r: 255, g: 0, b: 175},
    RGB8{r: 255, g: 0, b: 215},
    RGB8{r: 255, g: 0, b: 255},
    RGB8{r: 255, g: 95, b: 0},
    RGB8{r: 255, g: 95, b: 95},
    RGB8{r: 255, g: 95, b: 135},
    RGB8{r: 255, g: 95, b: 175},
    RGB8{r: 255, g: 95, b: 215},
    RGB8{r: 255, g: 95, b: 255},
    RGB8{r: 255, g: 135, b: 0},
    RGB8{r: 255, g: 135, b: 95},
    RGB8{r: 255, g: 135, b: 135},
    RGB8{r: 255, g: 135, b: 175},
    RGB8{r: 255, g: 135, b: 215},
    RGB8{r: 255, g: 135, b: 255},
    RGB8{r: 255, g: 175, b: 0},
    RGB8{r: 255, g: 175, b: 95},
    RGB8{r: 255, g: 175, b: 135},
    RGB8{r: 255, g: 175, b: 175},
    RGB8{r: 255, g: 175, b: 215},
    RGB8{r: 255, g: 175, b: 255},
    RGB8{r: 255, g: 215, b: 0},
    RGB8{r: 255, g: 215, b: 95},
    RGB8{r: 255, g: 215, b: 135},
    RGB8{r: 255, g: 215, b: 175},
    RGB8{r: 255, g: 215, b: 215},
    RGB8{r: 255, g: 215, b: 255},
    RGB8{r: 255, g: 255, b: 0},
    RGB8{r: 255, g: 255, b: 95},
    RGB8{r: 255, g: 255, b: 135},
    RGB8{r: 255, g: 255, b: 175},
    RGB8{r: 255, g: 255, b: 215},
    RGB8{r: 255, g: 255, b: 255},
    RGB8{r: 8, g: 8, b: 8},
    RGB8{r: 18, g: 18, b: 18},
    RGB8{r: 28, g: 28, b: 28},
    RGB8{r: 38, g: 38, b: 38},
    RGB8{r: 48, g: 48, b: 48},
    RGB8{r: 58, g: 58, b: 58},
    RGB8{r: 68, g: 68, b: 68},
    RGB8{r: 78, g: 78, b: 78},
    RGB8{r: 88, g: 88, b: 88},
    RGB8{r: 98, g: 98, b: 98},
    RGB8{r: 108, g: 108, b: 108},
    RGB8{r: 118, g: 118, b: 118},
    RGB8{r: 128, g: 128, b: 128},
    RGB8{r: 138, g: 138, b: 138},
    RGB8{r: 148, g: 148, b: 148},
    RGB8{r: 158, g: 158, b: 158},
    RGB8{r: 168, g: 168, b: 168},
    RGB8{r: 178, g: 178, b: 178},
    RGB8{r: 188, g: 188, b: 188},
    RGB8{r: 198, g: 198, b: 198},
    RGB8{r: 208, g: 208, b: 208},
    RGB8{r: 218, g: 218, b: 218},
    RGB8{r: 228, g: 228, b: 228},
    RGB8{r: 238, g: 238, b: 238},
];
